#include"SPI_Definitions.h"
#include "cc112x_registers.h"

char CCRF2_SpiCmdStrobe(char cmd){
    char rc;
    RF_CS = 0;           // TRXEM_SPI_BEGIN();
    asm NOP;
     while(RF_SO == 1);
    rc = SPI_Read(cmd);  //  TRXEM_SPI_TX(cmd);
                         //  TRXEM_SPI_WAIT_DONE();
                         //  rc = TRXEM_SPI_RX();
    RF_CS = 1;           // TRXEM_SPI_END();
    asm NOP;
    return(rc);
}
// ---------------------------------------

void CCRF2_ReadWriteBurstSingle(char addr,char *pData,unsigned len)
{
  unsigned i;
  /* Communicate len number of bytes: if RX - the procedure sends 0x00 to push bytes from slave*/
  if(addr & RADIO_READ_ACCESS)
  {
    if(addr & RADIO_BURST_ACCESS)
    {
      for (i = 0; i < len; i++)
      {
          *pData++ = SPI_Read(0);            /* Possible to combining read and write as one access type */
      }
    }
    else
    {
      *pData = SPI_Read(0);
    }
  }
  else
  {
    if(addr&RADIO_BURST_ACCESS)
    {
      /* Communicate len number of bytes: if TX - the procedure doesn't overwrite pData */
      for (i = 0; i < len; i++)
      {
        SPI_Write(*pData++);
      }
    }
    else
    {
      SPI_Write(*pData);
    }
  }
  return;
}

char CCRF2_8BitRegAccess(char accessType, char addrByte, char *pData, unsigned len)
{
  char readValue;

  /* Pull CS_N low and wait for SO to go low before communication starts */
  RF_CS = 0;
  asm NOP;
  while(RF_SO == 1);
  /* send register address byte */
  readValue = SPI_Read(accessType|addrByte);
  CCRF2_ReadWriteBurstSingle(accessType|addrByte, pData, len);
  asm NOP;
  RF_CS = 1;
  /* return the status byte value */
  return(readValue);
}

char CCRF2_16BitRegAccess(char accessType, char extAddr, char regAddr, char *pData, char len)
{
  char readValue;

  RF_CS = 0;
  asm NOP;
  while(RF_SO == 1);
  /* send extended address byte with access type bits set */
  readValue = SPI_Read(accessType|extAddr);
  SPI_Write(regAddr);
  /* Communicate len number of bytes */
  CCRF2_ReadWriteBurstSingle(accessType|extAddr, pData, len);
  asm NOP;
  RF_CS = 1;
  /* return the status byte value */
  return(readValue);
}

char CCRF2_SpiWriteReg(unsigned addr, char *pData, char len){
  char tempExt  = (char)(addr>>8);
  char tempAddr = (char)(addr & 0x00FF);
  char rc;

  /* Checking if this is a FIFO access - returns chip not ready */
  if((CC112X_SINGLE_TXFIFO <= tempAddr) && (tempExt == 0)) return STATUS_CHIP_RDYn_BM;

  /* Decide what register space is accessed */
  if(!tempExt){
    rc = CCRF2_8BitRegAccess((RADIO_BURST_ACCESS|RADIO_WRITE_ACCESS), tempAddr, pData, len);
  }
  else if (tempExt == 0x2F){
    rc = CCRF2_16BitRegAccess((RADIO_BURST_ACCESS|RADIO_WRITE_ACCESS), tempExt, tempAddr, pData, len);
  }
  return (rc);
}

char CCRF2_SpiReadReg(unsigned addr, char *pData, char len)
{
  char tempExt  = (char)(addr>>8);
  char tempAddr = (char)(addr & 0x00FF);
  char rc;

  /* Checking if this is a FIFO access -> returns chip not ready  */
  if((CC112X_SINGLE_TXFIFO<=tempAddr)&&(tempExt==0)) return STATUS_CHIP_RDYn_BM;

  /* Decide what register space is accessed */
  if(!tempExt)
  {
    rc = CCRF2_8BitRegAccess((RADIO_BURST_ACCESS|RADIO_READ_ACCESS),tempAddr,pData,len);
  }
  else if (tempExt == 0x2F)
  {
    rc = CCRF2_16BitRegAccess((RADIO_BURST_ACCESS|RADIO_READ_ACCESS),tempExt,tempAddr,pData,len);
  }
  return (rc);
}

char CCRF2_SpiWriteTxFifo(char *pData, char len)
{
  char rc;
  rc = CCRF2_8BitRegAccess(0x00,CC112X_BURST_TXFIFO, pData, len);
  return (rc);
}

char CCRF2_SpiReadRxFifo(char * pData, char len)
{
  char rc;
  rc = CCRF2_8BitRegAccess(0x00,CC112X_BURST_RXFIFO, pData, len);
  return (rc);
}

char ReadGPIO2(){
  return RF_GP2;
}