#include "ccrf2_objects.h"
#include "ccrf2_resources.h"
#include "built_in.h"


// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIOB_ODR.B1;
sbit DriveX_Right at GPIOB_ODR.B8;
sbit DriveY_Up at GPIOB_ODR.B9;
sbit DriveY_Down at GPIOB_ODR.B0;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1500;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton_Round *local_round_button;
TButton_Round *exec_round_button;
int round_button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;


void Init_ADC() {
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  Delay_ms(100);
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init_ILI9341_8bit(320, 240);

  TP_TFT_Init(320, 240, 8, 9);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TLabel                 lblInit;
char lblInit_Caption[12] = "Init status";

  TLabel                 lblSentC;
char lblSentC_Caption[16] = "NONE";

  TLine                  Line1;
  TLine                  Line2;
  TButton_Round          btnInit;
char btnInit_Caption[15] = "Init RF module";

  TButton_Round          btnSend;
char btnSend_Caption[12] = "Send packet";

  TButton_Round          btnSetRX;
char btnSetRX_Caption[17] = "Wait for packets";

  TLabel                 lblTitle;
char lblTitle_Caption[14] = "CCRF2 Example";

  TLabel                 Label1;
char Label1_Caption[19] = "EasyMx PRO for STM";

  TLabel                 Label2;
char Label2_Caption[15] = "www.mikroe.com";

  TLabel                 lblReceivedC;
char lblReceivedC_Caption[16] = "NONE";

  TLabel                 lblReceived;
char lblReceived_Caption[10] = "Received:";

  TLabel                 lblSent;
char lblSent_Caption[6] = "Sent:";

  TButton_Round          * const code Screen1_Buttons_Round[3]=
         {
         &btnInit,             
         &btnSend,             
         &btnSetRX             
         };
  TLabel                 * const code Screen1_Labels[8]=
         {
         &lblInit,             
         &lblSentC,            
         &lblTitle,            
         &Label1,              
         &Label2,              
         &lblReceivedC,        
         &lblReceived,         
         &lblSent              
         };
  TLine                  * const code Screen1_Lines[2]=
         {
         &Line1,               
         &Line2                
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.Buttons_RoundCount        = 3;
  Screen1.Buttons_Round             = Screen1_Buttons_Round;
  Screen1.LabelsCount               = 8;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.LinesCount                = 2;
  Screen1.Lines                     = Screen1_Lines;
  Screen1.ObjectsCount              = 13;


  lblInit.OwnerScreen     = &Screen1;
  lblInit.Order           = 0;
  lblInit.Left            = 200;
  lblInit.Top             = 77;
  lblInit.Width           = 50;
  lblInit.Height          = 15;
  lblInit.Visible         = 1;
  lblInit.Active          = 1;
  lblInit.Caption         = lblInit_Caption;
  lblInit.FontName        = Tahoma11x13_Regular;
  lblInit.Font_Color      = 0x39C7;
  lblInit.OnUpPtr         = 0;
  lblInit.OnDownPtr       = 0;
  lblInit.OnClickPtr      = 0;
  lblInit.OnPressPtr      = 0;

  lblSentC.OwnerScreen     = &Screen1;
  lblSentC.Order           = 1;
  lblSentC.Left            = 241;
  lblSentC.Top             = 119;
  lblSentC.Width           = 29;
  lblSentC.Height          = 15;
  lblSentC.Visible         = 0;
  lblSentC.Active          = 1;
  lblSentC.Caption         = lblSentC_Caption;
  lblSentC.FontName        = Tahoma11x13_Regular;
  lblSentC.Font_Color      = 0x39C7;
  lblSentC.OnUpPtr         = 0;
  lblSentC.OnDownPtr       = 0;
  lblSentC.OnClickPtr      = 0;
  lblSentC.OnPressPtr      = 0;

  Line1.OwnerScreen     = &Screen1;
  Line1.Order           = 2;
  Line1.First_Point_X   = 10;
  Line1.First_Point_Y   = 35;
  Line1.Second_Point_X  = 310;
  Line1.Second_Point_Y  = 35;
  Line1.Visible         = 1;
  Line1.Pen_Width       = 1;
  Line1.Color           = 0x0000;

  Line2.OwnerScreen     = &Screen1;
  Line2.Order           = 3;
  Line2.First_Point_X   = 10;
  Line2.First_Point_Y   = 224;
  Line2.Second_Point_X  = 310;
  Line2.Second_Point_Y  = 224;
  Line2.Visible         = 1;
  Line2.Pen_Width       = 1;
  Line2.Color           = 0x0000;

  btnInit.OwnerScreen     = &Screen1;
  btnInit.Order           = 4;
  btnInit.Left            = 13;
  btnInit.Top             = 70;
  btnInit.Width           = 130;
  btnInit.Height          = 29;
  btnInit.Pen_Width       = 1;
  btnInit.Pen_Color       = 0x0000;
  btnInit.Visible         = 1;
  btnInit.Active          = 1;
  btnInit.Transparent     = 1;
  btnInit.Caption         = btnInit_Caption;
  btnInit.TextAlign       = _taCenter;
  btnInit.FontName        = Tahoma16x19_Regular;
  btnInit.PressColEnabled = 1;
  btnInit.Font_Color      = 0x0000;
  btnInit.Gradient        = 1;
  btnInit.Gradient_Orientation = 0;
  btnInit.Gradient_Start_Color = 0xFFFF;
  btnInit.Gradient_End_Color = 0xC618;
  btnInit.Color           = 0xC618;
  btnInit.Press_Color     = 0xE71C;
  btnInit.Corner_Radius   = 3;
  btnInit.OnUpPtr         = 0;
  btnInit.OnDownPtr       = 0;
  btnInit.OnClickPtr      = btnInitOnClick;
  btnInit.OnPressPtr      = 0;

  btnSend.OwnerScreen     = &Screen1;
  btnSend.Order           = 5;
  btnSend.Left            = 13;
  btnSend.Top             = 112;
  btnSend.Width           = 130;
  btnSend.Height          = 29;
  btnSend.Pen_Width       = 1;
  btnSend.Pen_Color       = 0x0000;
  btnSend.Visible         = 1;
  btnSend.Active          = 1;
  btnSend.Transparent     = 1;
  btnSend.Caption         = btnSend_Caption;
  btnSend.TextAlign       = _taCenter;
  btnSend.FontName        = Tahoma16x19_Regular;
  btnSend.PressColEnabled = 1;
  btnSend.Font_Color      = 0x0000;
  btnSend.Gradient        = 1;
  btnSend.Gradient_Orientation = 0;
  btnSend.Gradient_Start_Color = 0xFFFF;
  btnSend.Gradient_End_Color = 0xC618;
  btnSend.Color           = 0xC618;
  btnSend.Press_Color     = 0xE71C;
  btnSend.Corner_Radius   = 3;
  btnSend.OnUpPtr         = 0;
  btnSend.OnDownPtr       = 0;
  btnSend.OnClickPtr      = btnSendOnClick;
  btnSend.OnPressPtr      = 0;

  btnSetRX.OwnerScreen     = &Screen1;
  btnSetRX.Order           = 6;
  btnSetRX.Left            = 13;
  btnSetRX.Top             = 154;
  btnSetRX.Width           = 130;
  btnSetRX.Height          = 29;
  btnSetRX.Pen_Width       = 1;
  btnSetRX.Pen_Color       = 0x0000;
  btnSetRX.Visible         = 1;
  btnSetRX.Active          = 1;
  btnSetRX.Transparent     = 1;
  btnSetRX.Caption         = btnSetRX_Caption;
  btnSetRX.TextAlign       = _taCenter;
  btnSetRX.FontName        = Tahoma16x19_Regular;
  btnSetRX.PressColEnabled = 1;
  btnSetRX.Font_Color      = 0x0000;
  btnSetRX.Gradient        = 1;
  btnSetRX.Gradient_Orientation = 0;
  btnSetRX.Gradient_Start_Color = 0xFFFF;
  btnSetRX.Gradient_End_Color = 0xC618;
  btnSetRX.Color           = 0xC618;
  btnSetRX.Press_Color     = 0xE71C;
  btnSetRX.Corner_Radius   = 3;
  btnSetRX.OnUpPtr         = 0;
  btnSetRX.OnDownPtr       = 0;
  btnSetRX.OnClickPtr      = btnSetRXOnClick;
  btnSetRX.OnPressPtr      = 0;

  lblTitle.OwnerScreen     = &Screen1;
  lblTitle.Order           = 7;
  lblTitle.Left            = 42;
  lblTitle.Top             = 9;
  lblTitle.Width           = 136;
  lblTitle.Height          = 25;
  lblTitle.Visible         = 1;
  lblTitle.Active          = 1;
  lblTitle.Caption         = lblTitle_Caption;
  lblTitle.FontName        = Tahoma19x23_Regular;
  lblTitle.Font_Color      = 0x0000;
  lblTitle.OnUpPtr         = 0;
  lblTitle.OnDownPtr       = 0;
  lblTitle.OnClickPtr      = 0;
  lblTitle.OnPressPtr      = 0;

  Label1.OwnerScreen     = &Screen1;
  Label1.Order           = 8;
  Label1.Left            = 12;
  Label1.Top             = 225;
  Label1.Width           = 105;
  Label1.Height          = 15;
  Label1.Visible         = 1;
  Label1.Active          = 1;
  Label1.Caption         = Label1_Caption;
  Label1.FontName        = Tahoma11x13_Regular;
  Label1.Font_Color      = 0x39C7;
  Label1.OnUpPtr         = 0;
  Label1.OnDownPtr       = 0;
  Label1.OnClickPtr      = 0;
  Label1.OnPressPtr      = 0;

  Label2.OwnerScreen     = &Screen1;
  Label2.Order           = 9;
  Label2.Left            = 222;
  Label2.Top             = 225;
  Label2.Width           = 87;
  Label2.Height          = 15;
  Label2.Visible         = 1;
  Label2.Active          = 1;
  Label2.Caption         = Label2_Caption;
  Label2.FontName        = Tahoma11x13_Regular;
  Label2.Font_Color      = 0xF800;
  Label2.OnUpPtr         = 0;
  Label2.OnDownPtr       = 0;
  Label2.OnClickPtr      = 0;
  Label2.OnPressPtr      = 0;

  lblReceivedC.OwnerScreen     = &Screen1;
  lblReceivedC.Order           = 10;
  lblReceivedC.Left            = 240;
  lblReceivedC.Top             = 162;
  lblReceivedC.Width           = 29;
  lblReceivedC.Height          = 15;
  lblReceivedC.Visible         = 0;
  lblReceivedC.Active          = 1;
  lblReceivedC.Caption         = lblReceivedC_Caption;
  lblReceivedC.FontName        = Tahoma11x13_Regular;
  lblReceivedC.Font_Color      = 0x39C7;
  lblReceivedC.OnUpPtr         = 0;
  lblReceivedC.OnDownPtr       = 0;
  lblReceivedC.OnClickPtr      = 0;
  lblReceivedC.OnPressPtr      = 0;

  lblReceived.OwnerScreen     = &Screen1;
  lblReceived.Order           = 11;
  lblReceived.Left            = 173;
  lblReceived.Top             = 162;
  lblReceived.Width           = 49;
  lblReceived.Height          = 15;
  lblReceived.Visible         = 0;
  lblReceived.Active          = 1;
  lblReceived.Caption         = lblReceived_Caption;
  lblReceived.FontName        = Tahoma11x13_Regular;
  lblReceived.Font_Color      = 0x39C7;
  lblReceived.OnUpPtr         = 0;
  lblReceived.OnDownPtr       = 0;
  lblReceived.OnClickPtr      = 0;
  lblReceived.OnPressPtr      = 0;

  lblSent.OwnerScreen     = &Screen1;
  lblSent.Order           = 12;
  lblSent.Left            = 177;
  lblSent.Top             = 119;
  lblSent.Width           = 26;
  lblSent.Height          = 15;
  lblSent.Visible         = 0;
  lblSent.Active          = 1;
  lblSent.Caption         = lblSent_Caption;
  lblSent.FontName        = Tahoma11x13_Regular;
  lblSent.Font_Color      = 0x39C7;
  lblSent.OnUpPtr         = 0;
  lblSent.OnDownPtr       = 0;
  lblSent.OnClickPtr      = 0;
  lblSent.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetRoundButton(index)         CurrentScreen->Buttons_Round[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawRoundButton(TButton_Round *Around_button) {
    if (Around_button->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_button->Transparent, Around_button->Press_Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_End_Color, Around_button->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_button->Transparent, Around_button->Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_Start_Color, Around_button->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_button->Pen_Color, Around_button->Pen_Width);
      TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
        Around_button->Left + Around_button->Width - 2,
        Around_button->Top + Around_button->Height - 2, Around_button->Corner_Radius);
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_HORIZONTAL);
      TFT_Write_Text_Return_Pos(Around_button->Caption, Around_button->Left, Around_button->Top);
      if (Around_button->TextAlign == _taLeft)
        TFT_Write_Text(Around_button->Caption, Around_button->Left + 4, (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taCenter)
        TFT_Write_Text(Around_button->Caption, (Around_button->Left + (Around_button->Width - caption_length) / 2), (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taRight)
        TFT_Write_Text(Around_button->Caption, Around_button->Left + (Around_button->Width - caption_length - 4), (Around_button->Top + (Around_button->Height - caption_height) / 2));
    }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short round_button_idx;
  TButton_Round *local_round_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short line_idx;
  TLine *local_line;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  round_button_idx = 0;
  label_idx = 0;
  line_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 8, 9);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (round_button_idx < CurrentScreen->Buttons_RoundCount) {
      local_round_button = GetRoundButton(round_button_idx);
      if (order == local_round_button->Order) {
        round_button_idx++;
        order++;
        DrawRoundButton(local_round_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (line_idx < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  round_button_order  = -1;
  label_order         = -1;
  //  Buttons with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->Buttons_RoundCount ; _object_count++ ) {
    local_round_button = GetRoundButton(_object_count);
    if (local_round_button->Active == 1) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  _object_count = -1;
  if (round_button_order >  _object_count )
    _object_count = round_button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_round_button   = 0;
  exec_label          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnPressPtr != 0) {
          exec_round_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Round Button
    case 1: {
      if (PressedObject != 0) {
        exec_round_button = (TButton_Round*)PressedObject;
        if ((exec_round_button->PressColEnabled == 1) && (exec_round_button->OwnerScreen == CurrentScreen)) {
          DrawRoundButton(exec_round_button);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons with Round Edges
    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnUpPtr != 0)
          exec_round_button->OnUpPtr();
        if (PressedObject == (void *)exec_round_button)
          if (exec_round_button->OnClickPtr != 0)
            exec_round_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_round_button   = 0;
  exec_label          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawRoundButton(exec_round_button);
        }
        PressedObject = (void *)exec_round_button;
        PressedObjectType = 1;
        if (exec_round_button->OnDownPtr != 0) {
          exec_round_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);
  TFT_BLED = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(239, 3700, 224, 3679);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
