#include "easy_link_reg_config.h"
#include "SPI_Functions.h"
#include "ccrf2_objects.h"

extern void updateLabel(TLabel *lbl, char *str);

#define PKTLEN                  5  // 1 < PKTLEN < 126

unsigned packetCounter   = 0 absolute 0x20000100;
unsigned receivedCounter = 0 absolute 0x20000102;
char buffer[PKTLEN+1] = {0};

void createPacket(char txBuffer[]) {
  int i;
  char tmp = 0xAA;
    buffer[0] = PKTLEN;                           // Length byte
    buffer[1] = (char) (packetCounter >> 8);     // MSB of packetCounter
    buffer[2] = (char)  packetCounter;           // LSB of packetCounter

    // Fill rest of buffer with random bytes
    for(i = 3; i < PKTLEN; i++) {
        buffer[i] = (i%2?tmp:~tmp);
    }
}

void runTX(){
char txt[25];
  // Update packet counter
  packetCounter++;
  WordToStr(packetCounter, txt);
  updateLabel(&lblSentC, &txt);
  // Create a random packet with PKTLEN + 2 byte packet
  // counter + n x random bytes
  createPacket(buffer);
  // Write packet to TX FIFO
  CCRF2_SpiWriteTxFifo(buffer, sizeof(buffer));
  // Strobe TX to send packet
  CCRF2_SpiCmdStrobe(CC112X_STX);
  // Wait for interrupt that packet has been sent.
  // (Assumes the GPIO connected to the radioRxTxISR function is
  // set to GPIOx_CFG = 0x06)
  while(ReadGPIO2() == 0){
    GPIOD_ODRbits.ODR0 = 0;  // Test line
  }
  while(ReadGPIO2() == 1){
    GPIOD_ODRbits.ODR0 = 0;  // Test line
  }
  GPIOD_ODRbits.ODR0 = 1;   // Test line
  // Clear semaphore flag
  //      packetSemaphore = ISR_IDLE;
  Delay_100ms();
}

#define RX_FIFO_ERROR       0x11
void setRXmode(){
  CCRF2_SpiCmdStrobe(CC112X_SRX);
}

void runRX(){
char txt[25];
unsigned char rxBytes;
char marcState;
unsigned receivedCnt = 0;
  if(ReadGPIO2()){
    while(ReadGPIO2());
  // Read number of bytes in RX FIFO
    CCRF2_SpiReadReg(CC112X_NUM_RXBYTES, &rxBytes, 1);
    // Check that we have bytes in FIFO
    if(rxBytes != 0) {
        // Read MARCSTATE to check for RX FIFO error
        CCRF2_SpiReadReg(CC112X_MARCSTATE, &marcState, 1);
        // Mask out MARCSTATE bits and check if we have a RX FIFO error
        if((marcState & 0x1F) == RX_FIFO_ERROR) {
            // Flush RX FIFO
            CCRF2_SpiCmdStrobe(CC112X_SFRX);
            updateLabel(&lblReceivedC, "ERROR");
        } else {
            // Read n bytes from RX FIFO
            CCRF2_SpiReadRxFifo(buffer, rxBytes);
            // Check CRC ok (CRC_OK: bit7 in second status byte)
            // This assumes status bytes are appended in RX_FIFO
            // (PKT_CFG1.APPEND_STATUS = 1)
            // If CRC is disabled the CRC_OK field will read 1
            if(buffer[rxBytes - 1] & 0x80) {
                setRXmode();
                // Update packet counter
                receivedCounter++;
                receivedCnt  = buffer[1]<<8;
                receivedCnt += buffer[2];
                WordToStr(receivedCnt, txt);
                updateLabel(&lblReceivedC, &txt);
            }
        }
    }
  }
}

void registerConfig(void) {
  unsigned char writeByte;
  unsigned i;
  // Reset radio
  CCRF2_SpiCmdStrobe(CC112X_SRES);

  // Write registers to radio
  for(i = 0; i < (sizeof(preferredSettings)/sizeof(registerSetting_t)); i++) {
    writeByte = preferredSettings[i].dat;
    CCRF2_SpiWriteReg(preferredSettings[i].addr, &writeByte, 1);
  }
}

#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2

void manualCalibration(void) {

    char original_fs_cal2;
    char calResults_for_vcdac_start_high[3];
    char calResults_for_vcdac_start_mid[3];
    char marcstate;
    char writeByte;

    // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    CCRF2_SpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);

    // 2) Start with high VCDAC (original VCDAC_START + 2):
    CCRF2_SpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
    writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
    CCRF2_SpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);

    // 3) Calibrate and wait for calibration to be done
    //   (radio back in IDLE state)
    CCRF2_SpiCmdStrobe(CC112X_SCAL);

    do {
        CCRF2_SpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
    } while (marcstate != 0x41);

    // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with
    //    high VCDAC_START value
    CCRF2_SpiReadReg(CC112X_FS_VCO2,
                     &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
    CCRF2_SpiReadReg(CC112X_FS_VCO4,
                     &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
    CCRF2_SpiReadReg(CC112X_FS_CHP,
                     &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);

    // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
    writeByte = 0x00;
    CCRF2_SpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);

    // 6) Continue with mid VCDAC (original VCDAC_START):
    writeByte = original_fs_cal2;
    CCRF2_SpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);

    // 7) Calibrate and wait for calibration to be done
    //   (radio back in IDLE state)
    CCRF2_SpiCmdStrobe(CC112X_SCAL);

    do {
        CCRF2_SpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
    } while (marcstate != 0x41);

    // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained
    //    with mid VCDAC_START value
    CCRF2_SpiReadReg(CC112X_FS_VCO2,
                     &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
    CCRF2_SpiReadReg(CC112X_FS_VCO4,
                     &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
    CCRF2_SpiReadReg(CC112X_FS_CHP,
                     &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);

    // 9) Write back highest FS_VCO2 and corresponding FS_VCO
    //    and FS_CHP result
    if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] >
        calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
        writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
    } else {
        writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
        writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
        CCRF2_SpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
    }
}

void configureCCRF2(){
  registerConfig();
  manualCalibration();
}